<?php
/*
 * @component jOpenSim
 * @copyright Copyright (C) 2020 FoTo50 http://www.jopensim.com/
 * @license GNU/GPL v2 http://www.gnu.org/licenses/gpl-2.0.html
 */
defined('_JEXEC') or die();

require_once(JPATH_COMPONENT_SITE.DIRECTORY_SEPARATOR.'models'.DIRECTORY_SEPARATOR.'opensim.php');

class opensimModelInterfaceTerminals extends OpenSimModelOpenSim {
// ########################################
// ########## terminal functions ##########
// ########################################

	public function terminal_identify($data) {
		$functionblock = "terminal";
		if($this->settings['addons_inworldauth'] == 0) {
			if($this->debug['terminal'] == "1") $this->simpledebugzeile("Terminal identify (but disabled in config) fired from ".$data['remoteip']." at line ".__LINE__." in ".__FILE__,$functionblock);
			exit; // nothing to do if the addon is disabled
		} else {
			if($this->debug['terminal'] == "1") $this->simpledebugzeile("Terminal identify fired from ".$data['remoteip']." at line ".__LINE__." in ".__FILE__,$functionblock);
		}
		$db = JFactory::getDBO();
		// first clean up old ident requests
		if($this->settings['addons_identminutes'] > 0) {
			$query = sprintf("DELETE FROM #__opensim_inworldident WHERE created < DATE_SUB(NOW(), INTERVAL %d MINUTE)",$this->settings['addons_identminutes']);
			$db->setQuery($query);
			$db->execute();
		}
		// first check if uuid has already a joomla relation
		$query = sprintf("SELECT joomlaID FROM #__opensim_userrelation WHERE opensimID = '%s'",$data['identKey']);
		$db->setQuery($query);
		$db->execute();
		$num_rows = $db->getNumRows();
		if($num_rows > 0) {
			$retval = JText::_('INWORLDALREADYIDENTIFIED');
		} else {
			// no relation existing, check if some inworld ident is prepared
			$query = sprintf("SELECT joomlaID FROM #__opensim_inworldident WHERE inworldIdent = '%s'",$data['identString']);
			$db->setQuery($query);
			$db->execute();
			$num_rows = $db->getNumRows();
			if($num_rows > 0) {
				$joomlaId = $db->loadResult();
				// this should actually never happen, but better check this as well:
				$query = sprintf("SELECT opensimID FROM #__opensim_userrelation WHERE joomlaID = '%d'",$joomlaId);
				$db->setQuery($query);
				$db->execute();
				$num_rows = $db->getNumRows();
				if($num_rows > 0) { // something went completely wrong here, hope this causes ppl to report a bug
					$retval = "Error while double check joomlaID ".$joomlaId." (could be a bug)! Please contact the Gridmanager and/or FoTo50 at the support forum at http://www.jopensim.com";
				} else { // Everything ok, inworld account identified
					// Update the relation table
					$query = sprintf("INSERT INTO #__opensim_userrelation (joomlaID,opensimID) VALUES ('%1\$d','%2\$s')",$joomlaId,$data['identKey']);
					$db->setQuery($query);
					$db->execute();
					// and delete from the ident table
					$query = sprintf("DELETE FROM #__opensim_inworldident WHERE inworldIdent = '%s'",$data['identString']);
					$db->setQuery($query);
					$db->execute();
					$retval = JText::_('IDENTIFYINWORLDSUCCESS');
				}
			} else { // no inworld ident found, give a proper message
				$retval = JText::_('IDENTIFYINWORLDFAILED');
			}
		}
		if($this->debug['terminal'] == "1") $this->debugzeile("Response for ".__FUNCTION__.":\n\n".$retval,$functionblock);
		return $retval;
	}

	public function terminal_register($remoteip) {
		$functionblock = "terminal";
		if($this->settings['addons_inworldauth'] == 0) {
			if($this->debug['terminal'] == "1") $this->simpledebugzeile("Terminal register (but disabled in config) fired from ".$remoteip." at line ".__LINE__." in ".__FILE__,$functionblock);
			return JText::_('IDENTIFYINWORLDDISABLED');
		} else {
			if($this->debug['terminal'] == "1") $this->simpledebugzeile("Terminal register fired from ".$remoteip." at line ".__LINE__." in ".__FILE__,$functionblock);
		}
		$terminalDescription	= JFactory::getApplication()->input->get('terminalDescription','','string');
		$terminalUrl			= JFactory::getApplication()->input->get('myurl','','raw');
		$regionString			= $_SERVER['HTTP_X_SECONDLIFE_REGION'];
		$locationString			= $_SERVER['HTTP_X_SECONDLIFE_LOCAL_POSITION'];
		$terminalKey			= $_SERVER['HTTP_X_SECONDLIFE_OBJECT_KEY'];
		$terminalName			= $_SERVER['HTTP_X_SECONDLIFE_OBJECT_NAME'];
		$region_suchmuster		= "/([^\(])*/";
		preg_match($region_suchmuster,$regionString,$treffer);
		$region = trim($treffer[0]);
		$location_suchmuster	= "/[^\d]*([\d\.]*)[^\d]*([\d\.]*)[^\d]*([\d\.]*)/";
		preg_match_all($location_suchmuster,$locationString,$treffer,PREG_SET_ORDER);
		$location_x = $this->roundoff($treffer[0][1],0);
		$location_y = $this->roundoff($treffer[0][2],0);
		$location_z = $this->roundoff($treffer[0][3],0);
		$db = JFactory::getDBO();
		$query = sprintf("SELECT staticLocation FROM #__opensim_terminals WHERE terminalKey = '%s'",$terminalKey);
		$db->setQuery($query);
		$db->execute();
		$num_rows = $db->getNumRows();
		if($num_rows > 0) {
			$terminalstatic = $db->loadResult();
			if($terminalstatic == "1") {
				$retval = "Terminal found, but not updated due to static setting!";
			} else {
				$query = sprintf("UPDATE #__opensim_terminals SET
									terminalName = '%s',
									terminalDescription = '%s',
									terminalUrl = '%s',
									region = '%s',
									location_x = '%d',
									location_y = '%d',
									location_z = '%d'
								WHERE
									terminalKey = '%s'",
					$terminalName,
					$terminalDescription,
					$terminalUrl,
					$region,
					$location_x,
					$location_y,
					$location_z,
					$terminalKey);
				$db->setQuery($query);
				$db->execute();
				$retval = "Terminal found and sucessfully updated!";
			}
		} else {
			$query = sprintf("INSERT INTO #__opensim_terminals
									(terminalName,terminalDescription,terminalKey,terminalUrl,region,location_x,location_y,location_z)
								VALUES
									('%s','%s','%s','%s','%s','%d','%d','%d')",
					$terminalName,
					$terminalDescription,
					$terminalKey,
					$terminalUrl,
					$region,
					$location_x,
					$location_y,
					$location_z);
			$db->setQuery($query);
			$db->execute();
			$retval = "Terminal sucessfully registered!";
		}
		if($this->debug['terminal'] == "1") $this->debugzeile("Response for ".__FUNCTION__.":\n\n".$retval,$functionblock);
		return $retval;
	}

	public function terminal_setState($remoteip) {
		$functionblock = "terminal";
		if($this->debug['terminal'] == "1") $this->simpledebugzeile("Terminal setState fired from ".$remoteip." at line ".__LINE__." in ".__FILE__,$functionblock);
		$terminalState	= JFactory::getApplication()->input->get('state');
		$terminalKey	= $_SERVER['HTTP_X_SECONDLIFE_OBJECT_KEY'];
		$query			= sprintf("UPDATE #__opensim_terminals SET active = '%d' WHERE terminalKey = '%s'",$terminalState,$terminalKey);
		$db				= JFactory::getDBO();
		$db->setQuery($query);
		$db->execute();
		if($terminalState == 1) $retval = JText::_('TERMINALSETVISIBLE');
		else $retval = JText::_('TERMINALSETINVISIBLE');
		if($this->debug['terminal'] == "1") $this->debugzeile("Response for ".__FUNCTION__.":\n\n".$retval,$functionblock);
		return $retval;
	}

}
?>